/*
 *                            COPYRIGHT
 *
 *  cschem - modular/flexible schematics editor - libcschem (core library)
 *  Copyright (C) 2019 Tibor 'Igor2' Palinkas
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.*
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/cschem
 *    lead developer: email to cschem (at) igor2.repo.hu
 *    mailing list: cschem (at) list.repo.hu (send "subscribe")
 */

#include "config.h"
#include <stdio.h>

#define TDL_DONT_UNDEF
#include "oidpath.h"
#include <genlist/gentdlist_impl.c>
#undef TDL_DONT_UNDEF
#include <genlist/gentdlist_undef.h>

#include "concrete.h"

int csch_oidpath_parse(csch_oidpath_t *dst, const char *src)
{
	const char *curr;
	char *next;

	if (*src != '/')
		return -1;
	dst->vt.used = 0;
	for(curr = src + 1; curr != NULL; curr = next) {
		long oid = strtol(curr, &next, 10);

		if (*next == '\0')       next = NULL;
		else if (*next == '/')   next++;
		else                     return -1;

		csch_vtoid_append(dst, oid);
	}
	return 0;
}

csch_chdr_t *csch_oidpath_resolve(csch_sheet_t *sheet, const csch_oidpath_t *path)
{
	csch_chdr_t *curr;
	int n;

	if (path->vt.used < 1)
		return NULL;

	/* Note: resolves absolute paths only */

	/* first oid must be 1 or 2 for the top group */
	if (path->vt.array[0] == CSCH_TOP_OID_DIRECT)
		curr = &sheet->direct.hdr;
	else if (path->vt.array[0] == CSCH_TOP_OID_INDIRECT)
		curr = &sheet->indirect.hdr;
	else
		return NULL;

	for(n = 1; n < path->vt.used; n++) {
		csch_cgrp_t *grp;

		if ((curr->type != CSCH_CTYPE_GRP) && (curr->type != CSCH_CTYPE_GRP_REF))
			return NULL;
		grp = (csch_cgrp_t *)curr;

		curr = htip_get(&grp->id2obj, path->vt.array[n]);
		if (curr == NULL)
			return NULL;
	}

	return curr;
}

void csch_oidpath_free(csch_oidpath_t *path)
{
	csch_vtoid_truncate(path, 0);
}


int csch_oidpath_to_str_append(gds_t *dst, const csch_oidpath_t *path)
{
	int n;
	for(n = 0; n < path->vt.used; n++) {
		char tmp[64];
		int len = sprintf(tmp, "/%ld", (long)path->vt.array[n]);
		gds_append_len(dst, tmp, len);
	}
	return path->vt.used;
}

char *csch_oidpath_to_str(const csch_oidpath_t *path)
{
	gds_t tmp;
	gds_init(&tmp);
	csch_oidpath_to_str_append(&tmp, path);
	return tmp.array;
}


void csch_oidpath_from_obj(csch_oidpath_t *dst, const csch_chdr_t *src)
{
	const csch_chdr_t *o;
	long len = 0;

	dst->vt.used = 0;
	for(o = src; o != NULL; o = (const csch_chdr_t *)o->parent) len++;
	for(o = src; o != NULL; o = (const csch_chdr_t *)o->parent)
		csch_vtoid_set(dst, --len, o->oid);
}

