/*
 *                            COPYRIGHT
 *
 *  cschem - modular/flexible schematics editor - libcschem (core library)
 *  Copyright (C) 2022 Tibor 'Igor2' Palinkas
 *
 *  (Supported by NLnet NGI0 PET Fund in 2022)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/cschem
 *    lead developer: email to cschem (at) igor2.repo.hu
 *    mailing list: cschem (at) list.repo.hu (send "subscribe")
 */


#include <libcschem/config.h>

#include <genvector/gds_char.h>

#include <libcschem/concrete.h>
#include <libcschem/plug_library.h>

#include <librnd/core/compat_misc.h>

static csch_lib_backend_t be_symlib_local;

static int symlib_local_map_local(rnd_hidlib_t *hl, csch_lib_t *root_dir, const csch_cgrp_t *indirect)
{
	htip_entry_t *e;
	for(e = htip_first(&indirect->id2obj); e != NULL; e = htip_next(&indirect->id2obj, e)) {
		csch_cgrp_t *grp = e->value;
		if ((grp->hdr.type == CSCH_CTYPE_GRP) && (grp->role == CSCH_ROLE_SYMBOL)) {
			gds_t nm = {0};
			csch_lib_t *newent;
			char tmp[64];

			TODO("append textual identifiers to nm");

			sprintf(tmp, ".%d",  grp->hdr.oid);
			gds_append_str(&nm, tmp);

			newent = csch_lib_alloc_append(&be_symlib_local, root_dir, nm.array, CSCH_SLIB_STATIC);
			newent->backend_data.lng[0] = grp->hdr.oid;
			/* do not free nm: name ownership passed on to newent */
		}

	}
	return 0;
}

static int symlib_local_load(csch_sheet_t *sheet, void *dst, csch_lib_t *src, const char *params)
{
	if (params != NULL)
		return -1;
	return -1;
}

static void symlib_local_free(csch_lib_t *src)
{
}


int pplg_check_ver_symlib_local(int ver_needed) { return 0; }

void pplg_uninit_symlib_local(void)
{

}

int pplg_init_symlib_local(void)
{
	be_symlib_local.name = "symlib_local";
	be_symlib_local.map_local = symlib_local_map_local;
	be_symlib_local.load = symlib_local_load;
	be_symlib_local.free = symlib_local_free;

	csch_lib_backend_reg(csch_lib_get_master("symbol", 1), &be_symlib_local);

	return 0;
}

